// Persistence of Vision Ray Tracer: Include File
// File : sunpos.inc
// Auth : Ingo Janssen
// Date : 15-09-1998
// Desc : gives the position of the sun at a given date,
//        time, and position on earth.
// ======================================================
// North is in the +Z direction, south is -Z.
//
// Find your local position at http://164.214.2.59/gns/html/index.html
//
// Local time at http://www.hilink.com.au/times/  +1 hour = +15 degrees
//
// Equations used here can be found at http://193.12.249.96/pausch/comp/tutorial.html
// not only for the sun but also for the moon, planets and other stuff up there.
//
// At http://www.crest.org/staff/ceg/sunangle/  there is a java calculator
// for the sunangle,rise & set times etc. Also some good (visual) explanation.
// ======================================================
//
// Declare the following variables in your scenefile, then include "sunpos.inc".
// Don't forget to put the object{Sun} in your scene, or it will be eternal night.
//
// Check variables and assign defaults:

    // Date (Year in four digits)
#ifndef (Year)    #declare Year= 1998;  #end
#ifndef (Month)   #declare Month= 6;    #end
#ifndef (Day)     #declare Day= 21;     #end

    // Time (24 hour format)
#ifndef (Hour)    #declare Hour= 12;    #end
#ifndef (Minutes) #declare Minutes= 37; #end

    // Meridian of your local timezone
    // in degrees (east = positive, west = negative)
#ifndef (Lstm)    #declare Lstm= 15;    #end

    // Position on earth, 
    // Lattitude in degrees.decimal, northern hemisphere = positive, southern = negative
    // Longitude in degrees.decimal, east = positive, west is negative
#ifndef (LAT)     #declare LAT= 52.266; #end
#ifndef (LONG)    #declare LONG= 6.9;   #end

// ======================================================
// Calculate:

#if (abs(LONG-Lstm)>30)
    #warning "\nWARNING: \nLongitude differs by more than 30 degrees from timezone meridian.\n"
    #warning concat("Local timezone meridian is:",str(Lstm,5,0),"\n")
    #warning concat("Longitude is:",str(LONG,5,0),"\n")
#end

    // Calculate universal time (UT)
#declare T= Hour+(Minutes/60);
#declare UT= T-Lstm/15;
#if (0>UT)
    #declare Day= Day-1;
    #declare UT= 24+UT;
#end
#if (UT>24)
    #declare Day= Day+1;
    #declare UT= UT-24;
#end

    // Amount of days to, or from, the year 2000
#declare d= 367*Year-int((7*int((Year+int((Month+9))/12)))/4)+int((275*Month)/9)+Day-730530+UT/24;

    // Longitude of perihelion (w), eccentricity (e)
#declare w= 282.9404+4.70935E-5*d;
#declare e= 0.016709-1.151E-9*d;

    // Mean anomaly (M), sun's mean longitude (L)
#declare M= 356.0470+0.9856002585*d;
#if (0<M<360)
    #declare M= M-floor(M/360)*360;
#end
#declare L= w+M;
#if (0<L<360)
    #declare L= L-floor(L/360)*360;
#end

    // Obliquity of the ecliptic, eccentric anomaly (E)
#declare oblecl= 23.4393-3.563E-7*d;
#declare E= M+(180/pi)*e*sin(radians(M))*(1+e*cos(radians(M)));

    // Sun's rectangular coordinates in the plane of ecliptic (A,B)
#declare A= cos(radians(E))-e;
#declare B= sin(radians(E))*sqrt(1-e*e);

    // Distance (r), true anomaly (V), longitude of the sun (lon)
#declare r= sqrt(A*A+B*B);
#declare V= degrees(atan2(radians(B),radians(A)));
#declare lon= V+w;
#if (0<lon<360)
    #declare lon= lon-floor(lon/360)*360;
#end

    // Calculate declination (Decl) and right ascension (RA)
#declare Decl= degrees(asin(sin(radians(oblecl))*sin(radians(lon))));
#declare RA= degrees(atan2(sin(radians(lon))*cos(radians(oblecl)),cos(radians(lon))))/15;

    // Greenwich meridian siderial time at 00:00 (GMST0),siderial time (SIDTIME), hour angle (HA)
#declare GMST0= L/15+12;
#declare SIDTIME= GMST0+UT+LONG/15;
#declare HA= (SIDTIME-RA)*15;

    // This is what we're looking for: Altitude & Azimuth
#declare Al= degrees(asin(sin(radians(LAT))*sin(radians(Decl))+cos(radians(LAT))*cos(radians(Decl))*cos(radians(HA))));
#declare Az= degrees(atan2(sin(radians(HA)),cos(radians(HA))*sin(radians(LAT))-tan(radians(Decl))*cos(radians(LAT))))+180;

    // A simple sun, since I was only interested
    // in the light, not the object.
#declare Sun= light_source { 
    <0, 0, 0>
    rgb <1, 1, 1>
    translate    <0, 0, r*1496000>
    rotate       <-Al, Az, 0>
}
